import numpy as np
import qml

def predict(X,alpha,indices,iquery,max_size,sigmas,cut_distance):
    N_train=alpha.shape[0]
    N_bin=alpha.shape[1]
    
    K=np.zeros(N_train)
    for itrain in range(N_train):
   #    if np.mod(itrain,50) == 0:
   #       print(itrain, 'kernel elements between query and training molecules calculated, ', N_train-itrain, 'remaining')
        Xt=X[indices[itrain]]
        Xq=X[indices[iquery]]
        Yt=np.zeros([1,max_size,5,max_size],dtype=float)
        Yq=np.zeros([1,max_size,5,max_size],dtype=float)
        Yt[0]=Xt
        Yq[0]=Xq
        tmp=qml.fchl.get_global_kernels(Yq,Yt,sigmas=sigmas,cut_distance=cut_distance)
        K[itrain]=tmp[0,0,0]

    Int_pred=np.zeros(N_bin)
    for i_bin in range(N_bin):
        Int_pred[i_bin]=np.dot(K,alpha[:,i_bin])

    return Int_pred



import matplotlib.pyplot as plt
from scipy import interpolate
import numpy as np

def plot_stem(lam,Int,label,filename):
    plt.stem(lam, Int, label=label, linewidth=1.0, linefmt='r-', markerfmt=' ')
    plt.legend()
    plt.xlabel('Wavelength (nm)', fontsize = 15)
    plt.ylabel('Oscillator strength (au)', fontsize = 15)
    plt.savefig(filename,bbox_inches='tight')
    plt.clf()
    return 1

def plot_smooth(lam,Int,label,filename):
    dx=(max(lam)-min(lam))/1000.0
    lam_smooth=np.zeros(1000)
    for i in range(0,1000):
        lam_smooth[i]=min(lam)+i*dx+dx/2
    f=interpolate.interp1d(lam, Int, kind='cubic')
    Int_smooth=f(lam_smooth)
    plt.plot(lam_smooth, Int_smooth, label=label, linewidth=1.0, color='blue')
    plt.legend()
    plt.xlabel('Wavelength (nm)', fontsize = 15)
    plt.ylabel('Oscillator strength (au)', fontsize = 15)
    plt.savefig(filename,bbox_inches='tight')
    plt.clf()
    return 1

def plot_stem_smooth(lam,Int1,Int2,label,filename):
    label1=label[0]
    label2=label[1]
    dx=(max(lam)-min(lam))/1000.0
    lam_smooth=np.zeros(1000)
    for i in range(0,1000):
        lam_smooth[i]=min(lam)+i*dx+dx/2
    f=interpolate.interp1d(lam, Int2, kind='cubic')
    Int_smooth=f(lam_smooth)
    plt.stem(lam, Int1, label=label1, linefmt='r-', markerfmt=' ')
    plt.plot(lam_smooth, Int_smooth, label=label2, linewidth=1.0, color='blue')
    plt.legend()
    plt.xlabel('Wavelength (nm)', fontsize = 15)
    plt.ylabel('Oscillator strength (au)', fontsize = 15)
    plt.savefig(filename,bbox_inches='tight')
    plt.clf()
    return 1

def plot_bar(lambda_min, Int, dlambda, label, filename):
    plt.bar(lambda_min, Int, width = dlambda, label=label, color='red', alpha=0.7, align='edge')
    plt.legend()
    plt.xlabel('Wavelength (nm)', fontsize = 15)
    plt.ylabel('Oscillator strength (au)', fontsize = 15)
    plt.savefig(filename,bbox_inches='tight')
    plt.clf()
    return 1

def plot_bar_compare(lambda_min, Int1, Int2, dlambda, label, filename):
    label1=label[0]
    label2=label[1]
    plt.bar(lambda_min, Int1, width = dlambda, label=label1,  color='red', alpha=0.2, align='edge', edgecolor='red')
    plt.bar(lambda_min, Int2, width = dlambda, label=label2,  alpha=1.0, align='edge', fill=False, edgecolor='red')
    plt.legend()
    plt.xlabel('Wavelength (nm)', fontsize = 15)
    plt.ylabel('Oscillator strength (au)', fontsize = 15)
    plt.savefig(filename,bbox_inches='tight')
    plt.clf()
    return 1





import numpy as np
import qml

def prepare_trainingdata(N_train,load_K,file_kernel,indices,lamd,X,Int_lam,sigmas,cut_distance,max_size):


    N_bin=len(Int_lam[0])

    K=np.zeros([N_train,N_train],dtype=float)
    if load_K:
        K = np.load(file_kernel)
    else:
        print('Calculating FCHL kernel elements'+'\n')
        for itrain in range(N_train):
            K[itrain,itrain]=1.0 + lamd
            if np.mod(itrain,10) == 0:
                print(itrain, 'rows calculated', N_train-itrain, 'remaining')
            for jtrain in range(itrain+1,N_train):
                Xt=X[indices[itrain]]
                Xq=X[indices[jtrain]]
                Yt=np.zeros([1,max_size,5,max_size],dtype=float)
                Yq=np.zeros([1,max_size,5,max_size],dtype=float)
                Yt[0]=Xt
                Yq[0]=Xq
                tmp=qml.fchl.get_global_kernels(Yq,Yt,sigmas=sigmas,cut_distance=cut_distance)
                K[itrain,jtrain]=tmp[0,0,0]
                K[jtrain,itrain]=K[itrain,jtrain]
        np.save(file_kernel, K)

    P=np.zeros([N_train,N_bin],dtype=float)

    for ibin in range(N_bin):
        for itrain in range(N_train):
            P[itrain,ibin]=Int_lam[indices[itrain],ibin]

    return K, P



import numpy as np
import qmlspectrum
import pandas as pd
'''
A module containing functions for various types of binning.
'''
def bin_spectra_uniform(spec_path, read_P, file_P, wavelength_min, wavelength_max, N_bin):
    '''
    A function for binning spectra using a uniform bin width
    '''
    spec_files=qmlspectrum.read_files(spec_path)
    if read_P:
        bin_spectra=np.load(file_P)
        N_file=bin_spectra.shape[0]-2
        Int_lam=bin_spectra[1:N_file,:]
        lambda_min=bin_spectra[N_file,:]
        dlambda=bin_spectra[N_file+1,:]
    else:
        print('binning spectra')

        lambda_min=[]
        lambda_max=[]
        dlambda = (wavelength_max - wavelength_min)/N_bin
        for i_bin in range(N_bin):
            lambda_min.append( (i_bin)*dlambda )
            lambda_max.append( (i_bin+1)*dlambda )

        N_file=len(spec_files)
        i_file=0
        Int_lam=np.zeros([N_file+2,N_bin])
        for spec_csv in spec_files:
            if np.mod(i_file, 100) == 0:
                print(i_file,' out of ', N_file, ' done')
            spec_data=pd.read_csv(spec_path+'/'+spec_csv, names=['wavelength_nm', 'osc_strength'])
            wavelength=np.array(spec_data['wavelength_nm'])
            f=np.array(spec_data['osc_strength'])
            for i_bin in range(N_bin):
                sum_f=0.0
                for i_state in range(f.shape[0]):
                    if wavelength[i_state] > lambda_min[i_bin] and wavelength[i_state] <= lambda_max[i_bin]:
                        sum_f=sum_f+f[i_state]
                Int_lam[i_file,i_bin]=sum_f
            i_file=i_file+1
        Int_lam[N_file,:]=lambda_min
        Int_lam[N_file+1,:]=dlambda
        np.save(file_P, Int_lam)
        print('data saved in ', file_P)

    return Int_lam, lambda_min, dlambda

def bin_spectra_nonuniform(spec_path, read_P, file_P, wavelength_min, wavelength_max, N_train, spec_den, N_state):
    '''
    A function for binning spectra using non-uniform bin widths
    '''
    spec_files = qmlspectrum.read_files(spec_path)
    if read_P:
        bin_spectra=np.load(file_P)
        N_file=bin_spectra.shape[0]-2
        N_bin=bin_spectra.shape[1]
        Int_lam=bin_spectra[1:N_file,:]
        lambda_min=bin_spectra[N_file,:]
        dlambda=bin_spectra[N_file+1,:]
    else:
        print('binning spectra')

        # RR: FIXME, 10 April 2022
        # Should this be done for every training set size, separately? Maybe this should be done for 
        # only 100 examples or so. 
        
        # RR: TODO, 10 April 2022
        # There may be cases when different molecules can have different number of states and we may want
        # to bin uniformly. 

        N_bin = int(N_state/spec_den)
        print (spec_path, read_P, file_P, wavelength_min, wavelength_max, N_train, spec_den, N_state)
        print (N_bin)

        all_wavelength = []
        for i in range(N_train):
            spec_data = pd.read_csv(spec_path+'/'+spec_files[i], names=['wavelength_nm', 'osc_strength'])
            wavelength = spec_data['wavelength_nm'].tolist()
            all_wavelength = all_wavelength + wavelength

        all_wavelength = pd.DataFrame(all_wavelength, columns=['wavelength_nm'])
        all_wavelength['bins'], bins = pd.qcut(all_wavelength['wavelength_nm'], q=N_bin, retbins=True, precision=6)

        lambda_min = []
        lambda_max = []
        dlambda = []
        print(bins)
        for i in range(N_bin):
            lambda_min.append(bins[i])
            lambda_max.append(bins[i+1])
            dlambda.append( bins[i + 1] - bins[i] )
            print(i, bins[i],bins[i+1])

        N_file=len(spec_files)
        i_file=0
        Int_lam=np.zeros([N_file+2,N_bin])
        for spec_csv in spec_files:
            if np.mod(i_file, 100) == 0:
                print(i_file,' out of ', N_file, ' done')
            spec_data=pd.read_csv(spec_path+'/'+spec_csv, names=['wavelength_nm', 'osc_strength'])
            wavelength=np.array(spec_data['wavelength_nm'])
            f=np.array(spec_data['osc_strength'])
            for i_bin in range(N_bin):
                sum_f=0.0
                for i_state in range(f.shape[0]):
                    if wavelength[i_state] > lambda_min[i_bin] and wavelength[i_state] <= lambda_max[i_bin]:
                        sum_f=sum_f+f[i_state]
                Int_lam[i_file,i_bin]=sum_f
            i_file=i_file+1
        Int_lam[N_file,:]=lambda_min
        Int_lam[N_file+1,:]=dlambda
        np.save(file_P, Int_lam)
        print('data saved in ', file_P)

    return Int_lam, lambda_min, dlambda, N_bin



import numpy as np
from scipy import linalg
from scipy.linalg import cho_factor, cho_solve

def linalg_solve(N_bin,K,P,solver='cholesky',file_train_prop='scr1',file_train_coef='scr2'):

    N_train=K.shape[0]

    alpha=np.zeros([N_train,N_bin]) 

    if solver == 'cholesky':
        Klow, low = cho_factor(K)
        for ibin in range(N_bin):
            alpha[:,ibin] = cho_solve((Klow, low), P[:,ibin])
    else:
        for ibin in range(N_bin):
            alpha[:,ibin] =linalg.solve(K, P[:,ibin])

    return alpha

    f = open(file_train_prop, 'w')
    for i_mol in range(N_train):
        for i_bin in range(N_bin):
            f.write(str(P[i_mol,i_bin])+' ')
        f.write('\n')
    f.close()

    f = open(file_train_coef, 'w')
    for i_mol in range(N_train):
        for i_bin in range(N_bin):
            f.write(str(alpha[i_mol,i_bin])+' ')
        f.write('\n')
    f.close()



import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

PACKAGE_NAME = 'qmlspectrum'
VERSION = '22.04.14'
AUTHOR = 'Raghunathan Ramakrishnan'
AUTHOR_EMAIL = 'raghu.rama.chem@gmail.com'
URL = 'https://github.com/raghurama123/qmlspectrum'
LICENSE = 'MIT License'
DESCRIPTION = 'A test-suite that uses the package qml for modeling continuous spectra'
LONG_DESCRIPTION = 'See https://github.com/raghurama123/qmlspectrum for documentation, datasets and example python scripts to use this test-suite'
LONG_DESC_TYPE = "text/markdown"
INSTALL_REQUIRES = [ 'qml', 'pandas', 'numpy', 'scipy', 'matplotlib' ]

setup(name=PACKAGE_NAME,
      version=VERSION,
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      url=URL,
      license=LICENSE,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      long_description_content_type=LONG_DESC_TYPE,
      install_requires=INSTALL_REQUIRES,
      packages=find_packages()
      )



import numpy as np

def confidence_score(fpred,fDFT):

    fpred=np.array(fpred) 
    fDFT=np.array(fDFT)

    df = np.abs(fpred/np.sum(fpred)-fDFT/np.sum(fDFT))

    error = np.sum(df)

    phi=100 * ( 1 - error )

    return phi



import numpy as np
import random

def shuffle(shuffling, load_indices, file_indices, N_mol):

    if shuffling:
        if load_indices:
            indices=np.zeros(N_mol,dtype=int)
            f=open(file_indices, 'r')
            indices=[]
            for line in f:
                string=line.split()[0]
                indices.append(int(string))
            f.close()
        else:
            indices=list( random.sample(range(0, N_mol), N_mol) )
            f = open(file_indices, 'w')
            for i_mol in range(N_mol):
                f.write("%06d"%(indices[i_mol])+'\n')
            f.close()

    else:
        if load_indices:
            indices=np.zeros(N_mol,dtype=int)
            f=open(file_indices, 'r')
            indices=[]
            for line in f:
                string=line.split()[0]
                indices.append(int(string))
            f.close()
        else:
            indices = list(i_mol for i_mol in range(N_mol))
            f = open(file_indices, 'w')
            for i_mol in range(N_mol):
                f.write("%06d"%(indices[i_mol])+'\n')
            f.close()


    return indices
